<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImportBase.php");
	
	/*
	 * ex. : 
	 <customer mail="100007936@test.fr">
        <name>DUPONT</name>
        <surname>Marcel</surname>
        <phone />
        <mobile />
        <memo />
        <date_add>2009-06-19</date_add>
    </customer>
	 */
	class PolarisLinkImport_customer extends PolarisLinkImportBase 
	{
		private $Genders = null;
		
		public function OnInit()
		{
			parent::OnInit();						
		}		

		/*
			Obtient la référence d'un XML
		*/
		protected function getReferenceFromData($data)
		{
			if (isset($data["mail"]))
				return $data["mail"];

			return null;
		}
		
		private function GetIdent($title, $autocreate = true)
		{			
			if ($title == null)
				$title = "";
			$title = trim($title);
			if ($title == "")
				return 0;
			
			if ($this->Genders == null)
			{
				$this->Genders = array();
				foreach (Gender::getGenders() as $gender)
					$this->Genders[$gender->id] = $gender->name;
			}
			
			$ctitle = mb_strtolower($title);
			
			foreach ($this->Genders as $id => $gender)
			{
				$cgender = mb_strtolower($gender);
				if ($cgender == $ctitle || $cgender."." == $ctitle || $cgender == $ctitle.".")
					return $id;
			}
			
			// Toujours là, il faut le créer
			if ($autocreate)
			{
				$gender = new Gender();
				$gender->name = $this->AutoFillEmptyLangValue($gender->name, $title);
				$gender->type = 2;
				$gender->save();
				$this->Genders = null;
				return $gender->id;
			}

			return null;
		}		
		
		private function RegisterCustomerMail($customer)
		{
			if (empty($customer->id))
				return;

			foreach ($this->SafeSQL(
				sprintf("SELECT (mail = HEX(AES_ENCRYPT('%s', sha1('%s'))) AND mail_len = '%s') AS identical FROM "._DB_PREFIX_."polarislink_mails WHERE id_customer = '%s'",
							pSQL($customer->email), sha1(_COOKIE_KEY_), strlen($customer->email), $customer->id)) as $item)
			{
				// Existe déjà ?				
				if ($item["identical"] != "1")
				{
					// On va remplacer l'association ...
					Db::GetInstance()->query(sprintf("DELETE FROM "._DB_PREFIX_."polarislink_mails WHERE id_customer = '%s'", $customer->id));
				}
			}
			
			// Si on arrive ici, alors on enregistre l'association
			Db::getInstance()->query(
									sprintf("INSERT INTO "._DB_PREFIX_."polarislink_mails (id_customer, mail, mail_len) VALUES ('%s', HEX(AES_ENCRYPT('%s', sha1('%s'))), '%s')", 
										$customer->id, pSQL($customer->email), sha1(_COOKIE_KEY_), strlen($customer->email)));			
		}
		
		/*
			Trouve un client avec son adresse mail
		*/
		public static function GetCustomerForMail($mail, $throwExceptionIfNotFound = false)
		{	
			$id = self::GetCustomerIdForMail($mail, $throwExceptionIfNotFound);
			if (!empty($id))
				return new Customer($id);
			
			return NULL;
		}
		
		/*
			Trouve un numéro de client avec son adresse mail
		*/
		public static function GetCustomerIdForMail($mail, $throwExceptionIfNotFound = false)
		{
			$db = Db::GetInstance();

			$results = $db->executeS('SELECT id_customer FROM `'._DB_PREFIX_.'customer` WHERE `email` = \''.pSQL($mail).'\'', true, false);			
			if ($results === false)
				throw new PrestaShopDatabaseException($db->getMsgError());

			if (count($results) > 0)
				return $results[0]['id_customer'];
		
			// Pas trouvé, on cherche dans la table des mails			
			$results = $db->executeS(sprintf("SELECT l.id_customer FROM "._DB_PREFIX_."polarislink_mails l LEFT JOIN "._DB_PREFIX_."customer c ON (c.id_customer = l.id_customer) WHERE c.id_customer IS NOT NULL AND mail = HEX(AES_ENCRYPT('%s', sha1('%s'))) AND mail_len = '%s'", pSQL($mail), sha1(_COOKIE_KEY_), strlen($mail)), true, false);
			if ($results === false)
				throw new PrestaShopDatabaseException($db->getMsgError());

			foreach ($results as $item)
				return $item["id_customer"];
			
			// arrivé jusqu'ici ? c'est que nous n'avons pas trouvé le client !
			if ($throwExceptionIfNotFound)
				throw new Exception(sprintf("impossible de trouver le client correspondant à l'adresse mail '%s'", $mail));
						
			return NULL;
		}
		
		protected function OnImportData($data)
		{
			if (!$this->Link->OPT_ALLOW_I_CUSTOMERS)
				return;	// Pas d'importation de client !
			
			// Avons nous la référence ?
			$mail = (string) $data["mail"];
			if (empty($mail))
				throw new Exception("aucun mail précisé pour le client");
			
			// Adresse mail non valide => pas d'importation !
			if (!Validate::isEmail($mail))
				return;
						
			// On localise le client
			$customer = self::GetCustomerForMail($mail);
			
			// Ok, si le client existe, il a été localisé
			$deleted = (int) $data["deleted"];
			if ($deleted)
			{
				/*// On doit supprimer le client
				if (!empty($customer))
				{
					$customer->delete();
					$this->Log($customer, "Suppression du client %s", $customer->email);
				}*/

				// Il est non productif de supprimer le client
				// Souvent le client est supprimé de Polaris pour cause de doublon d'adresse email, à la place, on ne fait rien !
				// Si le client veut se supprimer du site, il peut le faire directement depuis Prestashop.

				return;				
			}
			else
			{
				$lastname = checkCustomerLastName($data->lastname);				
				$firstname = checkCustomerFirstname($data->firstname);
				if (empty(trim($lastname))) $lastname = "INCONNU";
				if (empty(trim($firstname))) $firstname = "Inconnu";

				if (empty($customer))
				{					
					$customer = new Customer();
					$log = $this->NewLogCtx($customer, "Création du client %s", [ $mail ]);

					$customer->show_public_prices = 1;
					$customer->deleted = 0;
					$customer->is_guest = 0;
					$customer->newsletter = 0;
					$customer->passwd = md5(openssl_random_pseudo_bytes(64));


					$this->updProperty($log, $customer, "email", $mail);
					$this->updProperty($log, $customer, "lastname", $lastname);
					$this->updProperty($log, $customer, "firstname", $firstname);									

					$this->updProperty($log, $customer, "date_add", $data->date_add);

					$log->commit();					
					$customer->addGroups(array($this->Link->CustomersGroupID));										
				}				
				else
					$log = $this->NewLogCtx($customer, "Modification du client");
			
				$this->updProperty($log, $customer, "lastname", $lastname);
				$this->updProperty($log, $customer, "firstname", $firstname);
				
				if (count($data->title) > 0)
				{
					$gender = $this->GetIdent($data->title, $this->Link->OPT_ALLOW_I_GENDERS);
					if (!empty($gender))
						$this->updProperty($log, $customer, "id_gender", $gender);
				}
				
				$birthday = (string) $data->birthdate;
				if (empty($birthday))
					$birthday = "0000-00-00";
				$this->updProperty($log, $customer, "birthday", $birthday);

				if (isset($data->optin))
					$this->updProperty($log, $customer, "newsletter", empty($data->optin) ? "0" : "1");
				if (isset($data->optin3rd))
					$this->updProperty($log, $customer, "optin", empty($data->optin3rd) ? "0" : "1");

				$this->updOptProperty($log, $customer, "note", $data, "memo");
				
				$this->updProperty($log, $customer, "active", 1);

				if ($log->commit())
				{
					// Si l'entité a été sauvée, émulation du actionCustomerAccountUpdate
					Hook::exec('actionCustomerAccountUpdate', [
						'customer' => $customer,
					]);		
				}

				$this->RegisterCustomerMail($customer);
			}
		}
	}
	